@@ -21,37 +21,31 @@
 
 #include "kerncompat.h"
 #include "ctree.h"
+#include "delayed-ref.h"
 
 struct btrfs_trans_handle {
+	struct btrfs_fs_info *fs_info;
 	u64 transid;
 	u64 alloc_exclude_start;
 	u64 alloc_exclude_nr;
+	bool reinit_extent_tree;
+	unsigned int allocating_chunk:1;
+	u64 delayed_ref_updates;
 	unsigned long blocks_reserved;
 	unsigned long blocks_used;
 	struct btrfs_block_group_cache *block_group;
+	struct btrfs_delayed_ref_root delayed_refs;
+
 };
 
-static inline struct btrfs_trans_handle *
-btrfs_start_transaction(struct btrfs_root *root, int num_blocks)
-{
-	struct btrfs_fs_info *fs_info = root->fs_info;
-	struct btrfs_trans_handle *h = malloc(sizeof(*h));
-
-	BUG_ON(!h);
-	BUG_ON(root->commit_root);
-	BUG_ON(fs_info->running_transaction);
-	fs_info->running_transaction = h;
-	fs_info->generation++;
-	h->transid = fs_info->generation;
-	h->alloc_exclude_start = 0;
-	h->alloc_exclude_nr = 0;
-	h->blocks_reserved = num_blocks;
-	h->blocks_used = 0;
-	h->block_group = NULL;
-	root->last_trans = h->transid;
-	root->commit_root = root->node;
-	extent_buffer_get(root->node);
-	return h;
-}
+struct btrfs_trans_handle* btrfs_start_transaction(struct btrfs_root *root,
+		int num_blocks);
+int __commit_transaction(struct btrfs_trans_handle *trans,
+				struct btrfs_root *root);
+int commit_tree_roots(struct btrfs_trans_handle *trans,
+			     struct btrfs_fs_info *fs_info);
+int btrfs_commit_transaction(struct btrfs_trans_handle *trans,
+			     struct btrfs_root *root);
+void btrfs_abort_transaction(struct btrfs_trans_handle *trans, int error);
 
 #endif
